@@ -1,5 +1,5 @@
 ;; Ethiopic language utilities for Mule
-;; Copyright (C) 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995 Free Software Foundation, Inc.
 ;; This file is part of Mule (MULtilingual Enhancement of GNU Emacs).
 ;; This file contains Ethipic characters.
 
@@ -20,6 +20,7 @@
 
 ;;; 94.10.13 created for Mule Ver.2.1 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
 ;;; 94.12.27 modified for Mule Ver.2.2 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
+;;; 95.7.24 modified for Mule Ver.2.3 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
 
 ;;
 ;; ETHIOPIC UTILITY FUNCTIONS
@@ -34,7 +35,7 @@
 ;; To automatically convert Ethiopic text to SERA format when posting news,
 ;;   (add-hook 'news-inews-hook 'fidel-to-sera-mail)
 ;;
-;; If the filename ends in ".sera", editing will be done in file
+;; If the filename ends in ".sera", editing will be done in fidel
 ;; while file I/O will be done in sera.
 ;;
 ;; The following two functions have been removed.
@@ -49,63 +50,63 @@
   [
    nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil
    nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil nil
-;  SPC  !   "   #   $   %   &   '    (   )   *   +    ,     -    .     /
+;;; SP  !   "   #   $   %   &   '    (   )   *   +    ,     -    .     /
    nil nil nil nil nil nil nil ("") nil nil nil nil ("$(2$Q(B") nil ("$(2$P(B") nil
-;   0   1   2   3   4   5   6   7   8   9    :      ;     <   =   >   ?   @
+;;; 0   1   2   3   4   5   6   7   8   9    :      ;     <   =   >   ?   @
    nil nil nil nil nil nil nil nil nil nil ("$(2$S(B") ("$(2$R(B") nil nil nil nil nil
-; A
+;;; A
    ("$(2"V(B" (?2 "$(2#b(B"))
-; B
+;;; B
    ("$(2!F(B" (?e "$(2!A(B") (?u "$(2!B(B") (?i "$(2!C(B") (?a "$(2!D(B") (?E "$(2!E(B") (?o "$(2!G(B") (?| "$(2!F(B")
          (?W "$(2!H(B" (?a "$(2!H(B")
 	          (?e "$(2!F#L(B") (?u "$(2!F#M(B") (?i "$(2!F#N(B") (?E "$(2!F#P(B") (?' "$(2!F#M(B")))
-; C
+;;; C
    ("$(2"8(B" (?e "$(2"3(B") (?u "$(2"4(B") (?i "$(2"5(B") (?a "$(2"6(B") (?E "$(2"7(B") (?o "$(2"9(B") (?| "$(2"8(B")
          (?W "$(2":(B" (?a "$(2":(B")
 	          (?e "$(2"8#L(B") (?u "$(2"8#M(B") (?i "$(2"8#N(B") (?E "$(2"8#P(B") (?' "$(2"8#M(B")))
-; D
+;;; D
    ("$(2$0(B" (?e "$(2$+(B") (?u "$(2$,(B") (?i "$(2$-(B") (?a "$(2$.(B") (?E "$(2$/(B") (?o "$(2$1(B") (?| "$(2$0(B"))
-; E
+;;; E
    ("$(2"W(B" (?2 "$(2#c(B"))
-; F
+;;; F
    ("$(2"@(B" (?e "$(2";(B") (?u "$(2"<(B") (?i "$(2"=(B") (?a "$(2">(B") (?E "$(2"?(B") (?o "$(2"A(B") (?| "$(2"@(B")
          (?W "$(2"B(B" (?a "$(2"B(B")
 	          (?e "$(2"@#L(B") (?u "$(2"@#M(B") (?i "$(2"@#N(B") (?E "$(2"@#P(B") (?' "$(2"@#M(B")))
-; G
+;;; G
    ("$(2$>(B" (?e "$(2$9(B") (?u "$(2$:(B") (?i "$(2$;(B") (?a "$(2$<(B") (?E "$(2$=(B") (?o "$(2$?(B") (?| "$(2$>(B"))
-; H
+;;; H
    ("$(2$"(B" (?e "$(2#{(B") (?u "$(2#|(B") (?i "$(2#}(B") (?a "$(2#~(B") (?E "$(2$!(B") (?o "$(2$#(B") (?| "$(2$"(B"))
-; I
+;;; I
    ("$(2"X(B" (?2 "$(2#d(B"))
-; J
+;;; J
    ("$(2$7(B" (?e "$(2$2(B") (?u "$(2$3(B") (?i "$(2$4(B") (?a "$(2$5(B") (?E "$(2$6(B") (?o "$(2$8(B") (?| "$(2$7(B"))
-; K
+;;; K
    ("$(2"x(B" (?e "$(2"s(B") (?u "$(2"t(B") (?i "$(2"u(B") (?a "$(2"v(B") (?E "$(2"w(B") (?o "$(2"y(B") (?| "$(2"x(B")
          (?W "$(2"{(B" (?e "$(2"z(B") (?u "$(2"{(B") (?i "$(2"|(B") (?a "$(2"}(B") (?E "$(2"~(B")))
-; L
+;;; L
    ("$(2!&(B" (?e "$(2!!(B") (?u "$(2!"(B") (?i "$(2!#(B") (?a "$(2!$(B") (?E "$(2!%(B") (?o "$(2!'(B") (?| "$(2!&(B")
          (?W "$(2!((B" (?a "$(2!((B")
                   (?e "$(2!&#L(B") (?u "$(2!&#M(B") (?i "$(2!&#N(B") (?E "$(2!&#P(B") (?' "$(2!&#M(B")))
-; M
+;;; M
    ("$(2!.(B" (?e "$(2!)(B") (?u "$(2!*(B") (?i "$(2!+(B") (?a "$(2!,(B") (?E "$(2!-(B") (?o "$(2!/(B") (?| "$(2!.(B")
          (?W "$(2!0(B" (?a "$(2!0(B")
 	          (?e "$(2!.#L(B") (?u "$(2!.#M(B") (?i "$(2!.#N(B") (?E "$(2!.#P(B") (?' "$(2!.#M(B")))
-; N
+;;; N
    ("$(2!n(B" (?e "$(2!i(B") (?u "$(2!j(B") (?i "$(2!k(B") (?a "$(2!l(B") (?E "$(2!m(B") (?o "$(2!o(B") (?| "$(2!n(B")
          (?W "$(2!p(B" (?a "$(2!p(B")
 	          (?e "$(2!n#L(B") (?u "$(2!n#M(B") (?i "$(2!n#N(B") (?E "$(2!n#P(B") (?' "$(2!n#M(B")))
-; O
+;;; O
    ("$(2"Y(B" (?2 "$(2#e(B"))
-; P
+;;; P
    ("$(2$E(B" (?e "$(2$@(B") (?u "$(2$A(B") (?i "$(2$B(B") (?a "$(2$C(B") (?E "$(2$D(B") (?o "$(2$F(B") (?| "$(2$E(B"))
-; Q
+;;; Q
    ("$(2#2(B" (?e "$(2#-(B") (?u "$(2#.(B") (?i "$(2#/(B") (?a "$(2#0(B") (?E "$(2#1(B") (?o "$(2#3(B") (?| "$(2#2(B")
          (?W "$(2#5(B" (?e "$(2#4(B") (?u "$(2#5(B") (?i "$(2#6(B") (?a "$(2#7(B") (?E "$(2#8(B")))
-; R
+;;; R
    ("$(2!6(B" (?e "$(2!1(B") (?u "$(2!2(B") (?i "$(2!3(B") (?a "$(2!4(B") (?E "$(2!5(B") (?o "$(2!7(B") (?| "$(2!6(B")
          (?W "$(2!8(B" (?a "$(2!8(B")
 	          (?e "$(2!6#L(B") (?u "$(2!6#M(B") (?i "$(2!6#N(B") (?E "$(2!6#P(B") (?' "$(2!6#M(B")))
-; S
+;;; S
    ("$(2"P(B" (?e "$(2"K(B") (?u "$(2"L(B") (?i "$(2"M(B") (?a "$(2"N(B") (?E "$(2"O(B") (?o "$(2"Q(B") (?| "$(2"P(B")
          (?W "$(2"R(B" (?a "$(2"R(B")
                   (?e "$(2"P#L(B") (?u "$(2"P#M(B") (?i "$(2"P#N(B") (?E "$(2"P#P(B") (?' "$(2"P#M(B"))
@@ -115,29 +116,29 @@
 		 (?a "$(2"R(B")
 		 (?e "$(2#]#L(B") (?u "$(2#]#M(B") (?i "$(2#]#N(B") (?E "$(2#]#P(B") (?' "$(2#]#M(B"))))
 		
-; T
+;;; T
    ("$(2"0(B" (?e "$(2"+(B") (?u "$(2",(B") (?i "$(2"-(B") (?a "$(2".(B") (?E "$(2"/(B") (?o "$(2"1(B") (?| "$(2"0(B")
          (?W "$(2"2(B" (?a "$(2"2(B")
 	          (?e "$(2"0#L(B") (?u "$(2"0#M(B") (?i "$(2"0#N(B") (?E "$(2"0#P(B") (?' "$(2"0#M(B")))
-; U
+;;; U
    ("$(2"T(B" (?2 "$(2#`(B"))
-; V
+;;; V
    ("$(2!N(B" (?e "$(2!I(B") (?u "$(2!J(B") (?i "$(2!K(B") (?a "$(2!L(B") (?E "$(2!M(B") (?o "$(2!O(B") (?| "$(2!N(B")
          (?W "$(2!P(B" (?a "$(2!P(B")
 	          (?e "$(2!N#L(B") (?u "$(2!N#M(B") (?i "$(2!N#N(B") (?E "$(2!N#P(B") (?' "$(2!N#M(B")))
-; W
+;;; W
    ("$(2#M(B" (?e "$(2#L(B") (?u "$(2#M(B") (?i "$(2#N(B") (?a "$(2#O(B") (?E "$(2#P(B"))
-; X
+;;; X
    ("$(2#y(B" (?e "$(2#t(B") (?u "$(2#u(B") (?i "$(2#v(B") (?a "$(2#w(B") (?E "$(2#x(B") (?o "$(2#z(B") (?| "$(2#y(B"))
-; Y
+;;; Y
    ("$(2$)(B" (?e "$(2$$(B") (?u "$(2$%(B") (?i "$(2$&(B") (?a "$(2$'(B") (?E "$(2$((B") (?o "$(2$*(B") (?| "$(2$)(B"))
-; Z
+;;; Z
    ("$(2!~(B" (?e "$(2!y(B") (?u "$(2!z(B") (?i "$(2!{(B") (?a "$(2!|(B") (?E "$(2!}(B") (?o "$(2"!(B") (?| "$(2!~(B")
          (?W "$(2""(B" (?a "$(2""(B")
 	          (?e "$(2!~#L(B") (?u "$(2!~#M(B") (?i "$(2!~#N(B") (?E "$(2!~#P(B") (?' "$(2!~#M(B")))
-;   [   \   ]   ^   _
+;;; [   \   ]   ^   _
    nil nil nil nil nil
-; `
+;;; `
    ("`"
     (?e "$(2#_(B") (?u "$(2#`(B") (?U "$(2#`(B") (?i "$(2#a(B") (?a "$(2#b(B") (?A "$(2#b(B")
     (?E "$(2#c(B") (?I "$(2#d(B") (?o "$(2#e(B") (?O "$(2#e(B")
@@ -154,69 +155,69 @@
         (?W "$(2"c(B" (?e "$(2"b(B") (?u "$(2"c(B") (?i "$(2"d(B") (?a "$(2"e(B") (?E "$(2"f(B")))
     (?k "$(2#r(B"
         (?e "$(2#m(B") (?u "$(2#n(B") (?i "$(2#o(B") (?a "$(2#p(B") (?E "$(2#q(B") (?o "$(2#s(B") (?| "$(2#r(B")))
-; a
+;;; a
    ("$(2"S(B" (?2 "$(2#b(B"))
 
-; b
+;;; b
    ("$(2!F(B" (?e "$(2!A(B") (?u "$(2!B(B") (?i "$(2!C(B") (?a "$(2!D(B") (?E "$(2!E(B") (?o "$(2!G(B") (?| "$(2!F(B")
          (?W "$(2!H(B" (?a "$(2!H(B")
 	          (?e "$(2!F#L(B") (?u "$(2!F#M(B") (?i "$(2!F#N(B") (?E "$(2!F#P(B") (?' "$(2!F#M(B")))
-; c
+;;; c
    ("$(2!^(B" (?e "$(2!Y(B") (?u "$(2!Z(B") (?i "$(2![(B") (?a "$(2!\(B") (?E "$(2!](B") (?o "$(2!_(B") (?| "$(2!^(B")
          (?W "$(2!`(B" (?a "$(2!`(B")
 	          (?e "$(2!^#L(B") (?u "$(2!^#M(B") (?i "$(2!^#N(B") (?E "$(2!^#P(B") (?' "$(2!^#M(B")))
-; d
+;;; d
    ("$(2"((B" (?e "$(2"#(B") (?u "$(2"$(B") (?i "$(2"%(B") (?a "$(2"&(B") (?E "$(2"'(B") (?o "$(2")(B") (?| "$(2"((B")
          (?W "$(2"*(B" (?a "$(2"*(B")
 	          (?e "$(2"(#L(B") (?u "$(2"(#M(B") (?i "$(2"(#N(B") (?E "$(2"(#P(B") (?' "$(2"(#M(B")))
-; e
+;;; e
    ("$(2"S(B" (?2 "$(2#_(B") (?3 "$(2"Z(B"))
-; f
+;;; f
    ("$(2"@(B" (?e "$(2";(B") (?u "$(2"<(B") (?i "$(2"=(B") (?a "$(2">(B") (?E "$(2"?(B") (?o "$(2"A(B") (?| "$(2"@(B")
          (?W "$(2"B(B" (?a "$(2"B(B")
 	          (?e "$(2"@#L(B") (?u "$(2"@#M(B") (?i "$(2"@#N(B") (?E "$(2"@#P(B") (?' "$(2"@#M(B")))
-; g
+;;; g
    ("$(2#>(B" (?e "$(2#9(B") (?u "$(2#:(B") (?i "$(2#;(B") (?a "$(2#<(B") (?E "$(2#=(B") (?o "$(2#?(B") (?| "$(2#>(B")
          (?W "$(2#A(B" (?e "$(2#@(B") (?u "$(2#A(B") (?i "$(2#B(B") (?a "$(2#C(B") (?E "$(2#D(B")))
-; h
+;;; h
    ("$(2"`(B" (?e "$(2"[(B") (?u "$(2"\(B") (?i "$(2"](B") (?a "$(2"^(B") (?E "$(2"_(B") (?o "$(2"a(B") (?| "$(2"`(B")
          (?W "$(2"c(B" (?e "$(2"b(B") (?u "$(2"c(B") (?i "$(2"d(B") (?a "$(2"e(B") (?E "$(2"f(B"))
 	 (?2 "$(2#k(B" (?e "$(2#f(B") (?u "$(2#g(B") (?i "$(2#h(B") (?a "$(2#i(B") (?E "$(2#j(B") (?o "$(2#l(B")
 	          (?| "$(2#k(B")
 	          (?W "$(2"c(B" (?e "$(2"b(B") (?u "$(2"c(B") (?i "$(2"d(B") (?a "$(2"e(B") (?E "$(2"f(B"))))
-; i
+;;; i
    ("$(2"U(B" (?2 "$(2#a(B"))
-; j
+;;; j
    ("$(2$7(B" (?e "$(2$2(B") (?u "$(2$3(B") (?i "$(2$4(B") (?a "$(2$5(B") (?E "$(2$6(B") (?o "$(2$8(B") (?| "$(2$7(B"))
-; k
+;;; k
    ("$(2"l(B" (?e "$(2"g(B") (?u "$(2"h(B") (?i "$(2"i(B") (?a "$(2"j(B") (?E "$(2"k(B") (?o "$(2"m(B") (?| "$(2"l(B")
          (?W "$(2"o(B" (?e "$(2"n(B") (?u "$(2"o(B") (?i "$(2"p(B") (?a "$(2"q(B") (?E "$(2"r(B"))
 	 (?2 "$(2#r(B" (?e "$(2#m(B") (?u "$(2#n(B") (?i "$(2#o(B") (?a "$(2#p(B") (?E "$(2#q(B") (?o "$(2#s(B")
 	          (?| "$(2#r(B")))
-; l
+;;; l
    ("$(2!&(B" (?e "$(2!!(B") (?u "$(2!"(B") (?i "$(2!#(B") (?a "$(2!$(B") (?E "$(2!%(B") (?o "$(2!'(B") (?| "$(2!&(B")
          (?W "$(2!((B" (?a "$(2!((B")
                   (?e "$(2!&#L(B") (?u "$(2!&#M(B") (?i "$(2!&#N(B") (?E "$(2!&#P(B") (?' "$(2!&#M(B")))
-; m
+;;; m
    ("$(2!.(B" (?e "$(2!)(B") (?u "$(2!*(B") (?i "$(2!+(B") (?a "$(2!,(B") (?E "$(2!-(B") (?o "$(2!/(B") (?| "$(2!.(B")
          (?W "$(2!0(B" (?a "$(2!0(B")
 	          (?e "$(2!.#L(B") (?u "$(2!.#M(B") (?i "$(2!.#N(B") (?E "$(2!.#P(B") (?' "$(2!.#M(B")))
-; n
+;;; n
    ("$(2!f(B" (?e "$(2!a(B") (?u "$(2!b(B") (?i "$(2!c(B") (?a "$(2!d(B") (?E "$(2!e(B") (?o "$(2!g(B") (?| "$(2!f(B")
          (?W "$(2!h(B" (?a "$(2!h(B")
 	          (?e "$(2!f#L(B") (?u "$(2!f#M(B") (?i "$(2!f#N(B") (?E "$(2!f#P(B") (?' "$(2!f#M(B")))
-; o
+;;; o
    ("$(2"Y(B" (?2 "$(2#e(B"))
-; p
+;;; p
    ("$(2$L(B" (?e "$(2$G(B") (?u "$(2$H(B") (?i "$(2$I(B") (?a "$(2$J(B") (?E "$(2$K(B") (?o "$(2$M(B") (?| "$(2$L(B"))
-; q
+;;; q
    ("$(2#&(B" (?e "$(2#!(B") (?u "$(2#"(B") (?i "$(2##(B") (?a "$(2#$(B") (?E "$(2#%(B") (?o "$(2#'(B") (?| "$(2#&(B")
          (?W "$(2#)(B" (?e "$(2#((B") (?u "$(2#)(B") (?i "$(2#*(B") (?a "$(2#+(B") (?E "$(2#,(B")))
-; r
+;;; r
    ("$(2!6(B" (?e "$(2!1(B") (?u "$(2!2(B") (?i "$(2!3(B") (?a "$(2!4(B") (?E "$(2!5(B") (?o "$(2!7(B") (?| "$(2!6(B")
          (?W "$(2!8(B" (?a "$(2!8(B")
 	          (?e "$(2!6#L(B") (?u "$(2!6#M(B") (?i "$(2!6#N(B") (?E "$(2!6#P(B") (?' "$(2!6#M(B")))
-; s
+;;; s
    ("$(2"H(B" (?e "$(2"C(B") (?u "$(2"D(B") (?i "$(2"E(B") (?a "$(2"F(B") (?E "$(2"G(B") (?o "$(2"I(B") (?| "$(2"H(B")
          (?W "$(2"J(B" (?a "$(2"J(B")
 	          (?e "$(2"H#L(B") (?u "$(2"H#M(B") (?i "$(2"H#N(B") (?E "$(2"H#P(B") (?' "$(2"H#M(B"))
@@ -225,26 +226,26 @@
 		  (?W "$(2"J(B" (?a "$(2"J(B")
 		           (?e "$(2#V#L(B") (?u "$(2#V#M(B") (?i "$(2#V#N(B") (?E "$(2#V#P(B")
 			   (?' "$(2#V#M(B"))))
-; t
+;;; t
    ("$(2!V(B" (?e "$(2!Q(B") (?u "$(2!R(B") (?i "$(2!S(B") (?a "$(2!T(B") (?E "$(2!U(B") (?o "$(2!W(B") (?| "$(2!V(B")
          (?W "$(2!X(B" (?a "$(2!X(B")
 	          (?e "$(2!V#L(B") (?u "$(2!V#M(B") (?i "$(2!V#N(B") (?E "$(2!V#P(B") (?' "$(2!V#M(B")))
-; u
+;;; u
    ("$(2"T(B" (?2 "$(2#`(B"))
-; v
+;;; v
    ("$(2!N(B" (?e "$(2!I(B") (?u "$(2!J(B") (?i "$(2!K(B") (?a "$(2!L(B") (?E "$(2!M(B") (?o "$(2!O(B") (?| "$(2!N(B")
          (?W "$(2!P(B" (?a "$(2!P(B")
 	          (?e "$(2!N#L(B") (?u "$(2!N#M(B") (?i "$(2!N#N(B") (?E "$(2!N#P(B") (?' "$(2!N#M(B")))
-; w
+;;; w
    ("$(2#J(B" (?e "$(2#E(B") (?u "$(2#F(B") (?i "$(2#G(B") (?a "$(2#H(B") (?E "$(2#I(B") (?o "$(2#K(B") (?| "$(2#J(B")
          (?W "$(2#M(B" (?e "$(2#L(B") (?u "$(2#M(B") (?i "$(2#N(B") (?a "$(2#O(B") (?E "$(2#P(B")))
-; x
+;;; x
    ("$(2!>(B" (?e "$(2!9(B") (?u "$(2!:(B") (?i "$(2!;(B") (?a "$(2!<(B") (?E "$(2!=(B") (?o "$(2!?(B") (?| "$(2!>(B")
          (?W "$(2!@(B" (?a "$(2!@(B")
 	          (?e "$(2!>#L(B") (?u "$(2!>#M(B") (?i "$(2!>#N(B") (?E "$(2!>#P(B") (?' "$(2!>#M(B")))
-; y
+;;; y
    ("$(2$)(B" (?e "$(2$$(B") (?u "$(2$%(B") (?i "$(2$&(B") (?a "$(2$'(B") (?E "$(2$((B") (?o "$(2$*(B") (?| "$(2$)(B"))
-; z
+;;; z
    ("$(2!v(B" (?e "$(2!q(B") (?u "$(2!r(B") (?i "$(2!s(B") (?a "$(2!t(B") (?E "$(2!u(B") (?o "$(2!w(B") (?| "$(2!v(B")
          (?W "$(2!x(B" (?a "$(2!x(B")
 	          (?e "$(2!v#L(B") (?u "$(2!v#M(B") (?i "$(2!v#N(B") (?E "$(2!v#P(B") (?' "$(2!v#M(B")))
@@ -292,59 +293,66 @@ buffer is read-only."
       (if ascii-mode
 	  (cond
 
-	   ; ascii mode, pre != \ 
+	   ;; ascii mode, pre != \ 
 	   ((/= pre ?\\ ))
 
-	   ; ascii mode, pre = \, fol = !
+	   ;; ascii mode, pre = \, fol = !
 	   ((= fol ?!)
 	    (backward-delete-char 1)
 	    (delete-char 1)
 	    (setq ascii-mode nil
 		  hard (not hard)))
 
-	   ; hard ascii mode, pre = \, fol != !
+	   ;; hard ascii mode, pre = \, fol != !
 	   (hard)
 
-	   ; soft ascii mode, pre = \, fol = {\ _ * < > 0..9 ~}
+	   ;; soft ascii mode, pre = \, fol = {\ _ * < > 0..9 ~}
 	   ((or (backward-delete-char 1) ; always nil
 		(eobp)
 		(sera-to-fidel-backslash)))
 
-	   ; soft ascii mode, pre = \, fol = SPC
+	   ;; soft ascii mode, pre = \, fol = SPC
 	   ((= fol 32)
 	    (delete-char 1)
 	    (setq ascii-mode nil))
 
-	   ; soft ascii mode, pre = \, fol = .
+	   ;; soft ascii mode, pre = \, fol = .
 	   ((= fol ?.)
 	    (delete-char 1)
 	    (insert ?$(2$P(B))
 
-	   ; soft ascii mode, pre = \, fol = ,
+	   ;; soft ascii mode, pre = \, fol = ,
 	   ((= fol ?,)
 	    (delete-char 1)
 	    (insert ?$(2$Q(B))
 
-	   ; soft ascii mode, pre = \, fol = ;
+	   ;; soft ascii mode, pre = \, fol = ;
 	   ((= fol ?\;)
 	    (delete-char 1)
 	    (insert ?$(2$R(B))
 
-	   ; soft ascii mode, pre = \, fol = :
+	   ;; soft ascii mode, pre = \, fol = :
 	   ((= fol ?:)
 	    (delete-char 1)
 	    (insert ?$(2$S(B))
 
-	   ; soft ascii mode, pre = \, fol = others
+	   ;; soft ascii mode, pre = \, fol = others
 	   (t
 	    (setq ascii-mode nil)))
 
 	(cond
 
-	 ; ethio mode, pre != sera
+	 ;; very special: skip "<" to ">" (or "&" to ";") if in w3-mode
+	 ((and (boundp 'sera-being-called-by-w3)
+	       sera-being-called-by-w3
+	       (or (= pre ?<) (= pre ?&)))
+	  (search-forward (if (= pre ?<) ">" ";")
+			  nil 0))
+
+	 ;; ethio mode, pre != sera
 	 ((or (< pre ?') (> pre ?z)))
 
-	 ; ethio mode, pre != \ 
+	 ;; ethio mode, pre != \ 
 	 ((/= pre ?\\ )
 	  (setq table (aref sera-to-fidel-table pre))
 	  (while (setq table2 (cdr (assoc (following-char) table)))
@@ -355,31 +363,31 @@ buffer is read-only."
 		(delete-region start (point))
 		(insert (car table)))))
 
-	 ; ethio mode, pre = \, fol = !
+	 ;; ethio mode, pre = \, fol = !
 	 ((= fol ?!)
 	  (backward-delete-char 1)
 	  (delete-char 1)
 	  (setq ascii-mode t
 		hard (not hard)))
 
-	 ; hard ethio mode, pre = \, fol != !
+	 ;; hard ethio mode, pre = \, fol != !
 	 (hard)
 
-	 ; soft ethio mode, pre = \, fol = {\ _ * < > 0..9 ~}
+	 ;; soft ethio mode, pre = \, fol = {\ _ * < > 0..9 ~}
 	 ((or (backward-delete-char 1)	; always nil
 	      (eobp)
 	      (sera-to-fidel-backslash)))
 
-	 ; soft ethio mode, pre = \, fol = SPC
+	 ;; soft ethio mode, pre = \, fol = SPC
 	 ((= fol 32)
 	  (delete-char 1)
 	  (setq ascii-mode t))
 
-	 ; soft ethio mode, pre = \, fol = {. , ; : | ' `}
+	 ;; soft ethio mode, pre = \, fol = {. , ; : | ' `}
 	 ((memq fol '(?. ?, ?\; ?: ?| ?' ?`))
 	  (forward-char 1))
 
-	 ; soft ethio mode, pre = \, fol = others
+	 ;; soft ethio mode, pre = \, fol = others
 	 (t
 	  (setq ascii-mode t))))))
   (goto-char (point-min)))
@@ -428,7 +436,7 @@ Returns t if something has been processed."
 		(<= ch ?9))
       (delete-char 1)
 
-      ; count up following zeros
+      ;; count up following zeros
       (setq z 0)
       (while (= (following-char) ?0)
 	(delete-char 1)
@@ -436,25 +444,25 @@ Returns t if something has been processed."
 
       (cond
 
-       ; first digit is 10, 20, ..., or 90
+       ;; first digit is 10, 20, ..., or 90
        ((= (mod z 2) 1)
-	; (- ch 40) means ?1 -> 9, ?2 -> 10, etc.
+	;; (- ch 40) means ?1 -> 9, ?2 -> 10, etc.
 	(insert (aref [?$(2$`(B ?$(2$a(B ?$(2$b(B ?$(2$c(B ?$(2$d(B ?$(2$e(B ?$(2$f(B ?$(2$g(B ?$(2$h(B] (- ch ?1)))
 	(setq z (1- z)))
 
-       ; first digit is 2, 3, ..., or 9
+       ;; first digit is 2, 3, ..., or 9
        ((/= ch ?1)
 	(insert (aref [?$(2$X(B ?$(2$Y(B ?$(2$Z(B ?$(2$[(B ?$(2$\(B ?$(2$](B ?$(2$^(B ?$(2$_(B] (- ch ?2))))
 
-       ; single 1
+       ;; single 1
        ((= z 0)
 	(insert "$(2$W(B")))
 
-      ; 100
+      ;; 100
       (if (= (mod z 4) 2)
 	  (insert"$(2$i(B"))
 
-      ; 10000
+      ;; 10000
       (insert-char ?$(2$j(B (/ z 4)))))
 
 ;;;###autoload
@@ -484,7 +492,7 @@ For backward compatibility, \"<ethiopic>\" and \"<>\" can be used instead of
 
       (cond
 
-       ; with markers
+       ;; with markers
        ((re-search-forward "^<sera>\n" nil t)
 	(goto-char (match-beginning 0))
 	(while (re-search-forward "^<sera>\n" nil t)
@@ -503,7 +511,7 @@ For backward compatibility, \"<ethiopic>\" and \"<>\" can be used instead of
 	     (progn (end-of-line) (point))
 	     'ascii-start)))
 
-       ; backward compatibility
+       ;; backward compatibility
        ((re-search-forward "^<ethiopic>\n" nil t)
 	(goto-char (match-beginning 0))
 	(while (re-search-forward "^<ethiopic>\n" nil t)
@@ -522,7 +530,7 @@ For backward compatibility, \"<ethiopic>\" and \"<>\" can be used instead of
 	     (progn (end-of-line) (point))
 	     'ascii-start)))
 
-       ; interactive & no markers
+       ;; interactive & no markers
        (arg
 	(sera-to-fidel-region border (point-max))
 	(goto-char (point-min))
@@ -531,7 +539,7 @@ For backward compatibility, \"<ethiopic>\" and \"<>\" can be used instead of
 	     (point)
 	     (progn (end-of-line) (point))))))
 
-      ; adjust the rmail marker
+      ;; adjust the rmail marker
       (if (eq major-mode 'rmail-mode)
 	  (set-marker
 	   (aref rmail-message-vector (1+ rmail-current-message))
@@ -549,10 +557,10 @@ in Ethio start mode.  The markers will not be removed."
       (error ""))
   (save-excursion
     (goto-char (point-min))
-    (while (re-search-forward "^<sera>\n" nil t)
+    (while (re-search-forward "<sera>" nil t)
       (sera-to-fidel-region
        (point)
-       (if (re-search-forward "^</sera>\n" nil t)
+       (if (re-search-forward "</sera>" nil t)
 	   (match-beginning 0)
 	 (point-max))
        nil
@@ -665,7 +673,7 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
 	   (not (y-or-n-p "Buffer is read-only.  Force to convert? ")))
       (error ""))
 
-  ; user's preference in transcription
+  ;; user's preference in transcription
   (aset fidel-to-sera-map 144 (if ethio-use-tigrigna-style "e" "a"))
   (let ((i 160)
 	(x (if ethio-W-sixth-always
@@ -676,7 +684,7 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
       (setq i (+ i 12)
 	    x (cdr x))))
 
-  ; main conversion routine
+  ;; main conversion routine
   (let ((lonec nil) ; if lonec = t, previous char was a lone consonant.
 	(fidel nil) ; if fidel = t, previous char was a fidel.
 	(digit nil) ; if digit = t, previous char was an Ethiopic digit.
@@ -698,7 +706,7 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
 	     ((<= ch 326)
 	      (if ascii-mode
 		  (insert "\\ "))
-	      (if (and (memq ch '(144 145 146 147 148 150 151)) ;;  (auiAEoe3)
+	      (if (and (memq ch '(144 145 146 147 148 150 151)) ; (auiAEoe3)
 		       (or lonec
 			   (and ethio-quote-vowel-always
 				fidel)))
@@ -712,7 +720,7 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
 	     ;; punctuations and symbols
 	     ((or (< ch 336) (> ch 355))
 	      (if (and ascii-mode
-		       (memq ch '(329 330 331 332))) ;;  (.,;:)
+		       (memq ch '(329 330 331 332))) ; (.,;:)
 		  (insert "\\"))
 	      (insert (aref fidel-to-sera-map ch))
 	      (setq lonec nil
@@ -831,7 +839,7 @@ If the buffer contains at least one Ethiopic character,
 	       (progn (end-of-line) (point))
 	       'ascii-start))
 
-	  ; adjust the rmail marker
+	  ;; adjust the rmail marker
 	  (if (eq major-mode 'rmail-mode)
 	      (set-marker
 	       (aref rmail-message-vector (1+ rmail-current-message))
@@ -916,12 +924,12 @@ whose name ends in \".sera\"."
     (message "Modify vowel to: ")
     (if (null (setq vowel (memq (read-char) '(?e ?u ?i ?a ?E ?' ?o))))
 	(error "Not a valid vowel.")
-      ; ?e -> 0, ?u -> 1, ?i -> 2, ?a -> 3, ?E -> 4, ?' -> 5, ?o -> 6
+      ;; ?e -> 0, ?u -> 1, ?i -> 2, ?a -> 3, ?E -> 4, ?' -> 5, ?o -> 6
       (setq vowel (- 7 (length vowel))))
 
     (cond
 
-     ; 8-form consonant
+     ;; 8-form consonant
      ((<= ch 143)
       (setq base (* (/ ch 8) 8))
       (cond
@@ -938,7 +946,7 @@ whose name ends in \".sera\"."
 	      (cons (+ base 5)		; (+ base 5) = lone consonant
 		    (+ 231 vowel))))))	; 231 = We
 
-     ; 12-form consonant
+     ;; 12-form consonant
      ((<= ch 235)
       (setq ch (- ch 152)		; 152 = 12-form consonant offset
 	    base (* (/ ch 12) 12))
@@ -952,7 +960,7 @@ whose name ends in \".sera\"."
        (t
 	(error "Not a valid vowel."))))
 
-     ; 7-form consonant
+     ;; 7-form consonant
      (t					; 236 = 7-form consonant offset
       (setq newch (+ (* (/ (- ch 236) 7) 7) vowel 236))))
 
@@ -989,14 +997,49 @@ whose name ends in \".sera\"."
     (save-restriction
       (narrow-to-region begin end)
       (goto-char (point-min))
-      (while (re-search-forward "\\(\\cE\\)[ $(2$N$O(B]\\(\\cE\\)" nil t)
-	(cond
-	 ((= ch ?1)
-	  (replace-match "\\1 \\2" nil nil))
-	 ((= ch ?2)
-	  (replace-match "\\1$(2$N(B\\2" nil nil))
-	 (t
-	  (replace-match "\\1$(2$O(B\\2" nil nil)))))))
+
+      (cond
+
+       ((= ch ?1)
+
+	;; A double column space or an Ethiopic word separator is always
+	;; converted to an ASCII space.
+	(while (re-search-forward "[$(2$N$O(B]" nil t)
+	  (replace-match " " nil nil)))
+
+       ((= ch ?2)
+
+	;; An Ethiopic word separator is always converted to
+	;; a double column space.
+	(while (search-forward "$(2$O(B" nil t)
+	  (replace-match "$(2$N(B"))
+
+	(goto-char (point-min))
+
+	;; ASCII spaces are converted only if they are placed
+	;; between two Ethiopic characters.
+	(while (re-search-forward "\\(\\cE\\)\\( \\)\\( *\\cE\\)" nil t)
+
+	  ;; Converting the first ASCII space
+	  (replace-match "\\1$(2$N(B\\3")
+
+	  ;; A double column space is \cE, so going back to the just
+	  ;; converted double column space makes it possible to find
+	  ;; the following ASCII spaces.
+	  (goto-char (match-beginning 2))))
+
+       ((= ch ?3)
+
+	;; If more than one consecutive space (either ASCII or double
+	;; width) is found between two Ethiopic characters, the first
+	;; space will be converted to an Ethiopic word separator.
+	(let (pred succ)
+	  (while (re-search-forward "[ $(2$N(B]\\([ $(2$N(B]*\\)" nil t)
+	    (and (setq pred (char-before (match-beginning 0)))
+		 (= (char-leading-char pred) lc-ethio)
+		 (setq succ (char-after (match-end 0)))
+		 (= (char-leading-char succ) lc-ethio)
+		 (replace-match "$(2$O(B\\1" nil nil)))))))))
 
 ;;
 ;; special characters
@@ -1030,6 +1073,7 @@ whose name ends in \".sera\"."
 (define-key global-map [C-f5] 'fidel-to-sera-marker)
 (define-key global-map [f6] 'ethio-modify-vowel)
 (define-key global-map [f7] 'ethio-replace-space)
+(define-key global-map [S-f2] 'ethio-replace-space) ; as requested
 (define-key global-map [f8] 'ethio-input-special-character)
 
 (add-hook
