@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* 89.3.23  modified for Nemacs Ver.3.0 by K.Handa, S.Tomura */
 /* 89.9.12  modified for Nemacs Ver.3.2 by S.Tomura */
@@ -28,7 +29,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 struct Lisp_Process
   {
-    Lisp_Object_Int size;
+    EMACS_INT size;
     struct Lisp_Vector *v_next;
     /* Descriptor by which we read from this process */
     Lisp_Object infd;
@@ -37,6 +38,8 @@ struct Lisp_Process
     /* Descriptor for the tty which this process is using.
        nil if we didn't record it (on some systems, there's no need).  */
     Lisp_Object subtty;
+    /* Name of subprocess terminal.  */
+    Lisp_Object tty_name;
     /* Name of this process */
     Lisp_Object name;
     /* List of command arguments that this process was run with */
@@ -91,10 +94,8 @@ extern Lisp_Object Vprocess_alist;
    are waiting for it.  */
 extern int synch_process_alive;
 
-/* Communicate exit status of synch process to from sigchld_handler
+/* Communicate exit status of sync process to from sigchld_handler
    to Fcall_process.  */
-extern int synch_process_retcode;
-extern char *synch_process_death;
 
 /* Nonzero => this is a string explaining death of synchronous subprocess.  */
 extern char *synch_process_death;
